define battleship_rows = 10
define battleship_columns = battleship_rows

define free = " "
define banned = "."
define hit = "x"
define miss = "-"
define intact = "S"
define damage = "!"
define sunk = "#"

init python:
    class Ship (object):
        def __init__ (self, tag, size):
            self.tag = tag
            self.size = size
            self.hitpoints = size

        def __str__ (self):
            return self.tag

        def __repr__ (self):
            return "{}:{}:{}".format (self.tag, self.size, self.hitpoints)

        def hit (self):
            self.hitpoints -= 1
            return self.is_sunk ()

        def is_sunk (self):
            return self.hitpoints <= 0

    class battleship_AI (object):
        def __init__ (self, enemy, rows=battleship_rows, columns=None):
            self.enemy = enemy
            self.rows = rows
            self.columns = columns if columns else rows
            self.prio = []
            self.to_finish = {}

            for y in range (self.rows):
                row = []
                for x in range (self.columns):
                    row.append (5)
                self.prio.append (row)

        def neighbors (self, X, Y, eight=False):
            n = [(X, Y-1), (X-1, Y), (X+1, Y), (X, Y+1)]
            if eight:
                n.append ((X-1, Y-1))
                n.append ((X-1, Y+1))
                n.append ((X+1, Y-1))
                n.append ((X+1, Y+1))
            return [nx for nx in n if nx [0] >= 0 and nx [0] < self.columns and nx [1] >= 0 and nx [1] < self.rows]

        # If we hit (or sunk) something, there can't be OTHER ships near.
        # (Only parts of the same ship but that's handled seperately)
        # If we missed, tell the targetting alg. to spread out a bit.
        def update_prio (self, x, y, result):
            was_hit = result in [hit, sunk]
            for xn, yn in self.neighbors (x, y, eight=was_hit):
                if self.prio [yn][xn]:
                    if was_hit:
                        self.prio [yn][xn] = 0
                    else:
                        # This depriorizes neighbors of a spot we already tried.
                        # Thus avoid clustering.
                        self.prio [yn][xn] -= 1
            # Avoid shooting the same spot twice.
            self.prio [y][x] = 0

        def turn (self):
            if self.to_finish:
                check = renpy.random.choice (self.to_finish.keys ())
                x, y = self.to_finish.pop (check)
                # Possible results: None, miss, hit, sunk.
                result = self.enemy.incoming_shot (x, y)
                if result == hit:
                    # Hit means two things: we're not finished yet.
                    # But we do know the orientation now.
                    if check == "left":
                        self.to_finish.pop ("up", None)
                        self.to_finish.pop ("down", None)
                        if x > 0:
                            self.to_finish ["left"] = (x-1, y)
                    elif check == "right":
                        self.to_finish.pop ("up", None)
                        self.to_finish.pop ("down", None)
                        if x < self.columns-1:
                            self.to_finish ["right"] = (x+1, y)
                    elif check == "up":
                        self.to_finish.pop ("left", None)
                        self.to_finish.pop ("right", None)
                        if y > 0:
                            self.to_finish ["up"] = (x, y-1)
                    elif check == "down":
                        self.to_finish.pop ("left", None)
                        self.to_finish.pop ("right", None)
                        if y < self.rows-1:
                            self.to_finish ["down"] = (x, y+1)
                    else:
                        raise "What?"
                elif result == sunk:
                    self.to_finish = {}
                self.update_prio (x, y, result)
                return x, y, result

            highest = max ([p for row in self.prio for p in row])
            if highest == 0:
                raise "No more places to shoot at?"
            candidates = []
            for y in range (self.rows):
                candidates.extend ([(x, y) for x in range (self.columns) if self.prio [y][x] == highest])
            x, y = renpy.random.choice (candidates)
            result = self.enemy.incoming_shot (x, y)

            if result == hit:
                neighbors = self.neighbors (x, y)
                if (x-1, y) in neighbors:
                    self.to_finish ["left"] = (x-1, y)
                if (x+1, y) in neighbors:
                    self.to_finish ["right"] = (x+1, y)
                if (x, y-1) in neighbors:
                    self.to_finish ["up"] = (x, y-1)
                if (x, y+1) in neighbors:
                    self.to_finish ["down"] = (x, y+1)
            self.update_prio (x, y, result)
            return x, y, result

    class Board (object):
        def __init__ (self, is_player, rows=battleship_rows, columns=None, enemy=None):
            self.rows = rows
            self.columns = columns if columns else rows
            self.is_player = is_player
            self.ocean = []
            self.known = []
            self.ships = []
            self.brain = None if is_player else battleship_AI (enemy, self.rows, self.columns)

            for y in range (self.rows):
                row = []
                for x in range (self.columns):
                    row.append (free)
                self.ocean.append (row)

            for y in range (self.rows):
                row = []
                for x in range (self.columns):
                    row.append (free)
                self.known.append (row)

        def clear (self):
            for y in range (self.rows):
                for x in range (self.columns):
                    self.ocean [y][x] = free
                    self.known [y][x] = free
            self.ships = []

        def place (self, ship, X, Y, orientation):
            if X < 0 or Y < 0:
                return False
            if orientation == "horizontal":
                if X + ship.size > self.columns or Y >= self.rows:
                    return False
                for x in range (ship.size):
                    if self.ocean [Y][X+x] != free:
                        return False
                s = Ship (ship.tag, ship.size)
                for x in range (s.size):
                    self.ocean [Y][X+x] = s
                    if Y > 0:
                        self.ocean [Y-1][X+x] = banned
                    if Y < self.rows-1:
                        self.ocean [Y+1][X+x] = banned
                    if self.is_player:
                        self.known [Y][X+x] = intact
                if X > 0:
                    self.ocean [Y][X-1] = banned
                    if Y > 0:
                        self.ocean [Y-1][X-1] = banned
                    if Y < self.rows-1:
                        self.ocean [Y+1][X-1] = banned
                if X+s.size < self.columns:
                    self.ocean [Y][X+s.size] = banned
                    if Y > 0:
                        self.ocean [Y-1][X+s.size] = banned
                    if Y < self.rows-1:
                        self.ocean [Y+1][X+s.size] = banned
            else:
                if Y + ship.size > self.rows or X >= self.columns:
                    return False
                for y in range (ship.size):
                    if self.ocean [Y+y][X] != free:
                        return False
                s = Ship (ship.tag, ship.size)
                for y in range (s.size):
                    self.ocean [Y+y][X] = s
                    if X > 0:
                        self.ocean [Y+y][X-1] = banned
                    if X < self.columns-1:
                        self.ocean [Y+y][X+1] = banned
                    if self.is_player:
                        self.known [Y+y][X] = intact
                if Y > 0:
                    self.ocean [Y-1][X] = banned
                    if X > 0:
                        self.ocean [Y-1][X-1] = banned
                    if X < self.columns-1:
                        self.ocean [Y-1][X+1] = banned
                if Y+s.size < self.rows:
                    self.ocean [Y+s.size][X] = banned
                    if X > 0:
                        self.ocean [Y+s.size][X-1] = banned
                    if X < self.columns-1:
                        self.ocean [Y+s.size][X+1] = banned
            self.ships.append (s)
            return "You placed your {}".format (ship.tag)

        def randomly_place_fleet (self, fleet):
            for s in fleet:
                for x in xrange (1000):
                    X = renpy.random.randint (0, self.columns-1)
                    Y = renpy.random.randint (0, self.rows-1)
                    orientation = "vertical" if renpy.random.randint (0,1) else "horizontal"
                    if self.place (s, X, Y, orientation):
                        break
                else:
                    print "CAN'T PLACE THE {}!".format (s.tag)

        def print_ocean (self):
            for y in range (self.rows):
                row = self.ocean [y]
                print "".join ([row [x].tag [0] if isinstance (row [x], Ship) else row [x] for x in range (self.columns)])

        def check_placements (self):
            for s in self.ships:
                n = s.size
                for y in range (self.rows):
                    for x in range (self.columns):
                        if self.ocean [y][x] == s:
                            n -= 1
                if n != 0:
                    print s.tag, s.size, n
                    print self.ocean
                    self.print_ocean ()
                    print self.ships
                    raise "Wrong shipsize detected!"

        def lost (self):
            for s in self.ships:
                if not s.is_sunk ():
                    return False
            return True

        def incoming_shot (self, X, Y):
            if X < 0 or Y < 0 or X >= self.columns or Y >= self.rows:
                return None

            if not isinstance (self.ocean [Y][X], Ship):
                self.known [Y][X] = miss
                return miss

            # Found a ship. New hit.
            ret = hit

            if self.known [Y][X] in [free, intact]:
                # Let ship take a new hit a see if it sunk.
                if self.ocean [Y][X].hit ():
                    ret = sunk

            if self.is_player:
                if ret == sunk:
                    # Could be done more efficiently ... but what the hack.
                    for y in range (self.rows):
                        for x in range (self.columns):
                            if self.ocean [y][x] == self.ocean [Y][X]:
                                self.known [y][x] = sunk
                else:
                    self.known [Y][X] = damage
            else:
                self.known [Y][X] = hit
            return ret

    class Game (object):
        def __init__ (self, rows=battleship_rows, columns=None):
            self.rows = rows
            self.columns = columns if columns else rows
            self.player = Board (True, self.rows, self.columns)
            self.ai = Board (False, self.rows, self.columns, self.player)

        def finished (self):
            return self.player.lost () or self.ai.lost ()

        def parse (self, string):
            if string.lower () == "quit":
                return "quit"
            row = None
            col = None

            from string import ascii_lowercase, ascii_uppercase, digits
            for c in string:
                if c in digits:
                    if col == None:
                        col = c
                        continue
                    elif isinstance (col, basestring):
                        col += c
                        continue
                    else:
                        return "error"
                elif isinstance (col, basestring):
                    col = int (col)

                if c in ascii_lowercase and row == None:
                    row = ord (c) - ord ('a')
                elif c in ascii_uppercase and row == None:
                    row = ord (c) - ord ('A')
                elif c in ascii_uppercase and row == None:
                    row = ord (c) - ord ('A')
                elif c not in " .,:;-":
                    return "error"

            if row == None or row >= self.rows:
                return "error"
            if col == None:
                return "error"
            col = int(col)-1
            if col >= self.columns:
                return "error"

            return (col, row)
