# "Autumn breeze" palette in a more comprehensible form.
define autumn_breeze_palette = {
    "dark_petrol": "#2f3832",
    "petrol": "#3a4e48",
    "bright_petrol": "#999b6f",

    "dark_red": "#6f1d1b",
    "red": "#9e2828",
    "bright_red": "#cb6767",

    "dark_blue": "#41304a",
    "blue": "#6e5c7c",
    "bright_blue": "#b29cbc",

    "dark_slate": "#3d3b29",
    "slate": "#7c795c",
    "bright_slate": "#bcba9c",

    "dark_green": "#283e2a",
    "green": "#5c7c60",
    "bright_green": "#9cbca0",

    "black": "#1c110a",
    "eggshell": "#efe092",
    "skin": "#c8855f"
}

# Style properties for the block_shift_game_screen's UI elements.
# See https://www.renpy.org/dev-doc/html/screens.html#style-prefix
style battleship_main_screen_text:
    xalign 0.5
    yalign 0.5
    color autumn_breeze_palette ["eggshell"]

style battleship_main_screen_grid:
    xalign 1.0
    yalign 0.0

define battleship_main_screen_cell_width = 48
define battleship_main_screen_cell_height = battleship_main_screen_cell_width

style battleship_targetting_screen_text:
    xalign 0.5
    yalign 0.5
    # size 12
    color autumn_breeze_palette ["eggshell"]

style battleship_targetting_screen_grid:
    xalign 0.0
    yalign 0.0

define battleship_targetting_screen_cell_width = 48
define battleship_targetting_screen_cell_height = battleship_targetting_screen_cell_width


define symbols = {
    free: None,
    banned: "sunken_ship.png",
    hit: "red_pin.png",
    miss: "white_pin.png",
    intact: "ship.png",
    damage: "damaged_ship.png",
    sunk: "sunken_ship.png"
}


screen battleship_main_screen (side):
    $ board = side.known
    $ columns = side.columns
    $ rows = side.rows
    style_prefix "battleship_main_screen"

    grid rows+1 columns+1:
        # First line is only column headers, first column is only row headers. Thus the very first cell is empty.
        null width battleship_main_screen_cell_width height battleship_main_screen_cell_height

        # Now the column headers.
        for x in range (columns):
            text str(x+1)

        for y in range (rows):
            text chr(65+y)

            for x in range (columns):
                window:
                    xsize battleship_main_screen_cell_width
                    ysize battleship_main_screen_cell_height
                    if (x+y) % 2:
                        background "dark_tile.png"
                    else:
                        background "bright_tile.png"
                    if board [y][x] == free:
                        null
                    else:
                        add symbols [board [y][x]] xalign 0.5 yalign 0.5

screen battleship_targetting_screen (side):
    $ board = side.known
    $ columns = side.columns
    $ rows = side.rows
    style_prefix "battleship_targetting_screen"

    grid rows+1 columns+1:
        null width battleship_targetting_screen_cell_width height battleship_targetting_screen_cell_height

        for x in range (columns):
            text str(x+1)

        for y in range (rows):
            text chr(65+y)

            for x in range (columns):
                if board [y][x] == free:
                    if (x+y) % 2:
                        imagebutton idle "dark_tile.png":
                            xalign 0.5
                            yalign 0.5
                            xsize battleship_targetting_screen_cell_width
                            ysize battleship_targetting_screen_cell_height
                            action Return ((x, y))
                    else:
                        imagebutton idle "bright_tile.png":
                            xalign 0.5
                            yalign 0.5
                            xsize battleship_targetting_screen_cell_width
                            ysize battleship_targetting_screen_cell_height
                            action Return ((x, y))
                else:
                    window:
                        xsize battleship_targetting_screen_cell_width
                        ysize battleship_targetting_screen_cell_height
                        if (x+y) % 2:
                            background "dark_tile.png"
                        else:
                            background "bright_tile.png"
                        add symbols [board [y][x]] xalign 0.5 yalign 0.5

screen battleship_fleet_view (fleet, orientation):
        style_prefix "battleship_fleet"

        hbox:
            xalign 0.0
            yalign 0.0
            if orientation == "horizontal":
                window background "#b29cbc" xsize 400 ysize 400:
                    has vbox
                    spacing 30
                    for ship in fleet:
                        button action Return (ship):
                            hbox:
                                for i in range (ship.size):
                                    add symbols [intact]
            else:
                window background "#b29cbc" xsize 400 ysize 400:
                    has hbox
                    spacing 30
                    for ship in fleet:
                        button action Return (ship):
                            vbox:
                                for i in range (ship.size):
                                    add symbols [intact]
            vbox:
                textbutton "horizontal" action Return ("horizontal")
                textbutton "vertical" action Return ("vertical")
                null
                textbutton "Start over" action Return ("reset")
                textbutton "Quit game" action Return ("quit")

screen battleship_placement_screen (side):
    $ board = side.known
    $ columns = side.columns
    $ rows = side.rows
    style_prefix "battleship_main_screen"

    grid rows+1 columns+1:
        # First line is only column headers, first column is only row headers. Thus the very first cell is empty.
        null width battleship_main_screen_cell_width height battleship_main_screen_cell_height

        # Now the column headers.
        for x in range (columns):
            text str(x+1)

        for y in range (rows):
            text chr(65+y)

            for x in range (columns):
                if board [y][x] == intact:
                    add symbols [board [y][x]] xalign 0.5 yalign 0.5
                else:
                    if (x+y) % 2:
                        imagebutton idle "dark_tile.png":
                            xalign 0.5
                            yalign 0.5
                            xsize battleship_main_screen_cell_width
                            ysize battleship_main_screen_cell_height
                            action Return ((x, y))
                    else:
                        imagebutton idle "bright_tile.png":
                            xalign 0.5
                            yalign 0.5
                            xsize battleship_main_screen_cell_width
                            ysize battleship_main_screen_cell_height
                            action Return ((x, y))
